/**
 * Copyright (c) 2011, REMPL.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1) Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 2) Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution.
 * 3) Neither the name of the REMPL.com nor the names of its
 * contributors may be used to endorse or promote products derived
 * from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.rempl.aether;

import com.rempl.api.Artifact;
import com.rempl.api.Loader;
import com.rempl.api.LoadingException;
import com.rempl.api.Logger;
import com.rempl.api.ROM;

import java.io.File;
import java.net.URL;
import java.util.Collection;
import java.util.jar.JarFile;

import org.apache.commons.io.IOUtils;
import org.apache.commons.lang.StringUtils;

public abstract class AbstractAetherLoader implements Loader {

    /**
     * Load ROM.xml content from one of the JARs provided. We are
     * searching through the list of files, trying to get the
     * required <tt>rom.xml</tt> from any of them. If we fail to
     * find the file - exception is thrown.
     * @param jars List of JARs
     * @return Content of ROM.xml found (yes, we load it to memory)
     * @throws LoadingException If anything goes wrong
     * @see #load(Artifact)
     */
    protected abstract String rom(final Collection<File> jars)
        throws LoadingException;

    /**
     * Used to look up Artifacts in repository.
     * @return the resolver for artifacts
     */
    protected abstract Resolver getResolver();

    /**
     * Add the given urls to custom loader.
     * @param urls The URL array to be added
     */
    protected abstract void addToLoader(URL[] urls);

    /**
     * {@inheritDoc}
     */
    @Override
    public final void resolve(final Artifact artifact) throws LoadingException {
        // download all necessary JAR files through Maven
        this.classpath(this.jars(artifact));
        Logger.info(this, "#resolve(%s): resolved", artifact);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public final ROM load(final Artifact artifact) throws LoadingException {
        // download all necessary JAR files through Maven
        final Collection<File> jars = this.jars(artifact);
        // Find, unpack, and return contents of rom.xml found
        // in one of downloaded JARs.
        final String xml = this.rom(jars);
        ROM rom;
        try {
            rom = new ROM(xml);
        } catch (com.rempl.api.ROMException ex) {
            throw new LoadingException(ex);
        }
        // Add all downloaded JARs to the classpath in order
        // to enable instantiation of workers later through
        // ROM class instances.
        this.classpath(jars);
        Logger.info(this, "#load(%s): loaded", artifact);
        return rom;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public final Class forName(final String name) throws LoadingException {
        try {
            return Class.forName(
                name,
                true,
                Thread.currentThread().getContextClassLoader()
            );
        } catch (ClassNotFoundException ex) {
            throw new LoadingException(ex);
        }
    }

    /**
     * Resolves the artifact, and returns a full list of JARs
     * inside it (including dependencies). Pay attention that
     * the list may contain directories, not only JAR files!
     * @param artifact The artifact to resolve
     * @return Collection of JARs
     * @throws LoadingException If something goes wrong
     * @see #load(Artifact)
     * @see #resolve(Artifact)
     */
    protected final Collection<File> jars(final Artifact artifact)
        throws LoadingException {
        // Download all necessary JAR files through AETHER. It's
        // a special instrument to manipulate with Maven-like
        // repositories.
        final Collection<File> jars = this.getResolver().resolve(artifact);
        Logger.debug(
            this,
            "#jars(%s) found %d artifacts: %s",
            artifact,
            jars.size(),
            StringUtils.join(jars, ", ")
        );
        return jars;
    }


    /**
     * Extend current classpath with the list of JARs given. We
     * need to avoid duplicate JARs in the classpath, mostly for
     * the sake of performance.
     * @param jars List of JARs
     * @throws LoadingException If anything goes wrong
     * @see #load(Artifact)
     * @todo #15 This mechanism is ugly and has to be replaced with
     *           something better. The biggest problem with this
     *           current approach (besides performance) is that resources
     *           from child artifacts reload/replace resources from parent
     *           artifacts.
     */
    protected final void classpath(final Collection<File> jars)
        throws LoadingException {
        final URL[] urls = new URL[jars.size()];
        int idx = 0;
        for (File jar : jars) {
            try {
                urls[idx] = jar.toURI().toURL();
            } catch (java.net.MalformedURLException ex) {
                throw new LoadingException(ex);
            }
            idx += 1;
        }
        this.addToLoader(urls);
        Logger.debug(
            this,
            "#classpath(%d JARs): equals to %s",
            jars.size(),
            StringUtils.join(urls, ":")
        );
    }

    /**
     * Load ROM.xml content from JAR file.
     * @param jar The JAR file to look into
     * @return Content of ROM.xml found, or NULL if nothing found in the JAR
     * @throws com.rempl.api.LoadingException If anything goes wrong
     * @see #rom(java.util.Collection)
     */
    protected final String readRomFromJar(final File jar)
        throws LoadingException {
        try {
            final JarFile jarFile = new JarFile(jar.getPath());
            if (jarFile.getJarEntry(ROM.PATH) == null) {
                return null;
            }
            final URL url = new URL(
                "jar:file:"
                    + jar.getPath()
                    + "!/"
                    + ROM.PATH
            );
            final String xml = IOUtils.toString(url.openStream());
            Logger.debug(
                this,
                "#inJar(%s): rom.xml loaded (%d bytes)",
                url,
                xml.length()
            );
            return xml;
        } catch (java.net.MalformedURLException ex) {
            throw new LoadingException(ex);
        } catch (java.io.IOException ex) {
            throw new LoadingException(ex);
        }
    }
}
