/**
 * Copyright (c) 2011, REMPL.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1) Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 2) Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution.
 * 3) Neither the name of the REMPL.com nor the names of its
 * contributors may be used to endorse or promote products derived
 * from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.rempl.aether;

// API classes
import com.rempl.api.Artifact;
import com.rempl.api.LoadingException;
import com.rempl.api.Logger;

import com.rempl.commons.ProxyConfig;

// for file manipulations
import java.io.File;

// collections
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

// AETHER
import org.sonatype.aether.RepositorySystem;
import org.sonatype.aether.repository.Proxy;
import org.sonatype.aether.repository.RemoteRepository;
import org.sonatype.aether.resolution.ArtifactResult;
import org.sonatype.aether.resolution.DependencyRequest;
import org.sonatype.aether.resolution.DependencyResolutionException;

/**
 * Loader of an artifact into home directory of the executer.
 * @see <a href="https://docs.sonatype.org/display/AETHER/Home">Aether</a>
 */
public final class Aether implements Resolver {

    /**
     * Resolve the artifact and return a list of JARs.
     * @param artifact The name of the artifact to find
     * @return The collection of files found
     * @throws LoadingException If something goes wrong
     */
    @Override
    public Collection<File> resolve(final Artifact artifact)
        throws LoadingException {
        final Collection<ArtifactResult> found = this.artifacts(
            artifact.toString());
        final Collection<File> jars = new ArrayList<File>();
        for (ArtifactResult aresult : found) {
            Logger.debug(
                this,
                "%s resolved to %s",
                aresult.getArtifact(),
                aresult.getArtifact().getFile()
            );
            jars.add(aresult.getArtifact().getFile());
        }
        return jars;
    }

    /**
     * Return a list of found artifacts.
     * @param artifact The name of the artifact to find
     * @return The collection of artifacts found
     * @throws LoadingException If something goes wrong
     * @see #resolve(String)
     */
    private Collection<ArtifactResult> artifacts(final String artifact)
        throws LoadingException {
        final RepositorySystem repo = RepositoryBuilder.INSTANCE.build();
        final DependencyRequest request = RequestBuilder.INSTANCE.build(
            artifact);
        final ProxyConfig proxyConfig = new ProxyConfig();
        if (proxyConfig.getHost() != null) {
            this
                .addProxyInfo(request.getCollectRequest().getRepositories(),
                proxyConfig);
        }
        try {
            return repo.resolveDependencies(
                SessionBuilder.INSTANCE.build(repo), request
            ).getArtifactResults();
        } catch (DependencyResolutionException ex) {
            throw new LoadingException(ex);
        }
    }

    /**
     * Add proxy to remote repositories.
     *
     * @param repositories repository list to add proxy info
     * @param proxyConfig proxy data that will be added
     */
    private void addProxyInfo(final List<RemoteRepository> repositories,
        final ProxyConfig proxyConfig) {
        for (RemoteRepository repo : repositories) {
            repo.setProxy(new Proxy(Proxy.TYPE_HTTP, proxyConfig.getHost(),
                proxyConfig.getPort(), null));
        }
    }

}
